#!/bin/bash
#
######################################################################
#
#	set_OfficePrefs_2019_365.sh
#
#	Samuel Herschbein
#	sam@SamIsIT.com
#	sherschbein@me.com
#
#	Revision History
#	2020-03-11 - Original.
#
#	This script sets several Microsoft Office 2019/365 privacy and data collection preferences. It also sets
#		two AutoUpdate preferences: how it checks for updates and which Required Data Notices are displayed.
#
#	The script displays usage help. Pass -h for a summary and -hd for details.
#
#	The 16.28 update for August 2019 removed the GUI for several important privacy preferences. Now they must be set
#		using defaults write, a configuration Profile, MDM, et al.
#
#	Microsoft's web site states that Office 2019/365 does not upload any document content. Previously Microsoft stated that
#		"Send full diagnostic data" could upload memory dumps containing content from open documents.
#	To comply with United States' HIPAA rules no PHI (Personal Health Information) can be uploaded to Microsoft.
#	Given Microsoft's history on factual disclosures concerning privacy and data collection, it's safest to minimize
#		what's uploaded to Microsoft.
#
#	A while back Microsoft changed "Send xxx diagnostic data" from a single preference for all Office apps to
#		an individual preference for each app. The Office 2019/365 16.28 update for August 2019 changed
#		this back to a single preference.
#
#	Some previous Office updates have reset preferences to the defaults. I've seen "Send basic diagnostic data" get
#		reset to the default "Send full diagnostic data". To be safe, run this script after every Office app update.
#
#	This script requires Office 2019/365 version 16.28, which is the August 11, 2019 update.
#
#	If -other or -all is specified along with -manual two additional AutoUpdate preferences are set for everyone but the current user:
# 		EnableCheckForUpdatesButton is set to false, this prevents other users from updating if AutoUpdate is launched.
#		StartDaemonOnAppLaunch is set to false, this prevents the AutoUpdate daemon from launching when Office apps are launched.
#
#	To keep the script succinct bash's one-line if-then and if-then-else else constructs are used:
#		test && ifTrue || ifFalse
#	I find the more concise script easier to debug in spite of the more obtuse construct. YMMV.
#
######################################################################
#
usageSummary="\nset_OfficePrefs_2019_365.sh usage summary: \n"
usageSummary+="\t[ -h ] Display this usage summary.\n"
usageSummary+="\t[ -hd ] Display usage details.\n"
usageSummary+="\t[ -me | -other | -all ] Which users' preferences get set.\n"
usageSummary+="\t[ -basic | -full | -zero ] What diagnostic data gets sent to Microsoft.\n"
usageSummary+="\t[ -contrue | -confalse ] Whether Connected Office Experiences (COE) are enabled.\n"
usageSummary+="\t[ -acptrue | -acpfalse ] Whether COE Analyzing Content is enabled.\n"
usageSummary+="\t[ -dcptrue | -dcpfalse ] Whether COE Downloading Content is enabled.\n"
usageSummary+="\t[ -opttrue | -optfalse ] Whether Optional COE are enabled.\n"
usageSummary+="\t[ -manual | -check | -install ] How AutoUpdate checks for updates.\n"
usageSummary+="\t[ -reqonly | -reqopt ] What AutoUpdate's Required Data Notice displays.\n"
usageSummary+="\n\tOptions that are not specified default to:\n\t\t-all -zero -confalse -acpfalse -dcpfalse -optfalse -install -reqonly\n"
usageSummary+="\tWhen -other or -all is specified the script must be run with sudo so it can modify other users.\n"
usageSummary+="\n\tTo minimize the data sent to Microsoft use: -zero -confalse\n\n"
#
usageDetails="\nset_OfficePrefs_2019_365.sh usage details:\n"
usageDetails+="\t[ -h ] Display usage summary.\n"
usageDetails+="\t[ -hd ] Display these usage details.\n\n"
usageDetails+="\t[ -me | -other | -all ] Which users' preferences get set:\n"
usageDetails+="\t\t-me: just the current user.\n"
usageDetails+="\t\t-other: all users except Guest and the current user.\n"
usageDetails+="\t\t-all: all users except Guest.\n"
usageDetails+="\n\t[ -basic | -full | -zero ] What diagnostic data gets sent to Microsoft:\n"
usageDetails+="\t\t-basic - basic diagnostic information (aka Required).\n"
usageDetails+="\t\t-full - full diagnostic information (aka Optional).\n"
usageDetails+="\t\t-zero - no diagnostic information (aka Neither).\n"
usageDetails+="\n\tConnected Office Experiences:\n"
usageDetails+="\t\t[ -contrue | -confalse ] Whether Connected Office Experiences (COE) are enabled:\n"
usageDetails+="\t\t\t-contrue - true (enabled)\n"
usageDetails+="\t\t\t-confalse - false (disabled)\n"
usageDetails+="\n\t\t[ -acptrue | -acpfalse ] Whether COE Analyzing Content is enabled:\n"
usageDetails+="\t\t\t-acptrue - true (enabled)\n"
usageDetails+="\t\t\t-acpfalse - false (disabled)\n"
usageDetails+="\n\t\t[ -dcptrue | -dcpfalse ] Whether COE Downloading Content is enabled:\n"
usageDetails+="\t\t\t-dcptrue - true (enabled)\n"
usageDetails+="\t\t\t-dcpfalse - false (disabled)\n"
usageDetails+="\n\t\t[ -opttrue | -optfalse ] Whether Optional COE are enabled:\n"
usageDetails+="\t\t\t-opttrue - true (enabled)\n"
usageDetails+="\t\t\t-optfalse - false (disabled)\n"
usageDetails+="\n\t[ -manual | -check | -install ] How AutoUpdate checks for updates:\n"
usageDetails+="\t\t-manual: manually check\n"
usageDetails+="\t\t-check: automatically check\n"
usageDetails+="\t\t-install: automatically check and install\n"
usageDetails+="\n\t[ -reqonly | -reqopt ] What AutoUpdate's Required Data Notice displays:\n"
usageDetails+="\t\t-reqonly: required data only\n"
usageDetails+="\t\t-reqopt: required and optional data\n"
usageDetails+="\n\tOptions that are not specified default to:\n\t\t-all -zero -confalse -acpfalse -dcpfalse -optfalse -install -reqonly\n"
usageDetails+="\tWhen -other or -all is specified the script must be run with sudo so it can modify other users.\n"
usageDetails+="\tTo minimize the data sent to Microsoft use: -zero -confalse\n"
usageDetails+="\tFor details on the preferences and their plist keys & values see:\n"
usageDetails+="\t\thttps://docs.microsoft.com/en-us/deployoffice/privacy/mac-privacy-preferences\n\n"
#
#	Check for help first.
[[ "$1" == "-h" ]] && { printf "$usageSummary"; exit; }
[[ "$1" == "-hd" ]] && { printf "$usageDetails"; exit; }
#
#	Preset all options to "" before parsing.
setWho=""
diagnosticData=""
conOfficeExpers=""
analyzingContent=""
downloadingContent=""
optConExpers=""
MAU_HowToCheck=""
MAU_ReqDataNotice=""
#
#	Parse the options. To report errors the variables are first populated with the option.
for option in ${1+"$@"}; do
	if [[ "$option" == "" ]]; then
		printf "\nBlank options are not allowed.\n$usageSummary"; exit
	elif [[ "$option" == "-me" || "$option" == "-other" || "$option" == "-all" ]]; then
		[[ "$setWho" == "" ]] && setWho="$option" || { printf "\n$option conflicts with the option $setWho already specified.\n$usageSummary"; exit; }
	elif [[ "$option" == "-basic" || "$option" == "-full" || "$option" == "-zero" ]]; then
		[[ "$diagnosticData" == "" ]] && diagnosticData="$option" || { printf "\n$option conflicts with the option $diagnosticData already specified.\n$usageSummary"; exit; }
	elif [[ "$option" == "-contrue" || "$option" == "-confalse" ]]; then
		[[ "$conOfficeExpers" == "" ]] && conOfficeExpers="$option" || { printf "\n$option conflicts with the option $conOfficeExpers already specified.\n$usageSummary"; exit; }
	elif [[ "$option" == "-acptrue" || "$option" == "-acpfalse" ]]; then
		[[ "$analyzingContent" == "" ]] && analyzingContent="$option" || { printf "\n$option conflicts with the option $analyzingContent already specified.\n$usageSummary"; exit; }
	elif [[ "$option" == "-dcptrue" || "$option" == "-dcpfalse" ]]; then
		[[ "$downloadingContent" == "" ]] && downloadingContent="$option" || { printf "\n$option conflicts with the option $downloadingContent already specified.\n$usageSummary"; exit; }
	elif [[ "$option" == "-opttrue" || "$option" == "-optfalse" ]]; then
		[[ "$optConExpers" == "" ]] && optConExpers="$option" || { printf "\n$option conflicts with the option $optConExpers already specified.\n$usageSummary"; exit; }
	elif [[ "$option" == "-manual" ||  "$option" == "-check" || "$option" == "-install" ]]; then
		[[ "$MAU_HowToCheck" == "" ]] && MAU_HowToCheck="$option" || { printf "\n$option conflicts with the option $MAU_HowToCheck already specified.\n$usageSummary"; exit; }
	elif [[ "$option" == "-reqonly" || "$option" == "-reqopt" ]]; then
		[[ "$MAU_ReqDataNotice" == "" ]] && MAU_ReqDataNotice="$option" || { printf "\n$option conflicts with the option $MAU_ReqDataNotice already specified.\n$usageSummary"; exit; }
	else
		printf "\nInvalid option: \"$option\"\n$usageSummary"; exit
	fi
done
#
#	Before setting unspecified options, check if sudo was needed.
#	This is done now so the error messages are more logical.
#
#	Get rid of the leading dash/hyphen so the text is readable in printf.
[[ "$setWho" != "" ]] && setWho="${setWho:1:${#setWho}}"
#
#	Set flag for running as root.
[[ "$(whoami)" == "root" ]] && runningAsRoot=true || runningAsRoot=false
#	When setting users other than the current user the script must be running as root (sudo).
if [[ "$setWho" == "" || "$setWho" == "other" || "$setWho" == "all" ]] && ! $runningAsRoot; then
	[[ "$setWho" == "" ]] && setWho="all"
	printf "\nThis script must be run using sudo when setting $setWho users.\n"
	if [[ "$#" -eq 0 ]]; then
		printf "\t(No options were specified, script assumed -all for which users\' preferences get set.)\n"
	fi
	printf "$usageSummary"
	exit
fi
#	Get the current user's username.
$runningAsRoot && myusername="$(logname)" || myusername="$(whoami)"
#
#	Set unspecified options to defaults.
[[ "$setWho" == "" ]] && setWho="all"								#	Exception, setWho had its leading - removed above.
[[ "$diagnosticData" == "" ]] && diagnosticData="-zero"
[[ "$conOfficeExpers" == "" ]] && conOfficeExpers="-confalse"
[[ "$MAU_HowToCheck" == "" ]] && MAU_HowToCheck="-install"
[[ "$MAU_ReqDataNotice" == "" ]] && MAU_ReqDataNotice="-reqonly"
#
#	Make sure the Connected Office Experiences settings jibe with each other.
if [[ "$conOfficeExpers" == "-confalse" ]]; then
	if [[ "$analyzingContent" == "-acptrue" || "$downloadingContent" == "-dcptrue" || "$optConExpers" == "-opttrue" ]]; then
		printf "\nOptions -acptrue, -dcptrue, and -opttrue cannot be used with the option -confalse.\n$usageSummary"; exit
	else
		analyzingContent="false"		#	These must be false, they weren't set to true, so don't bother checking if already false.
		downloadingContent="false"
		optConExpers="false"
	fi
fi
#
#	Finally, all the options are valid.
#
#	Check for Office 2019/365 >= 16.28 by checking Word's version. Word is included with all versions of Office 2019/365.
wordVersion="$(defaults read /Applications/Microsoft\ Word.app/Contents/Info CFBundleShortVersionString 2>&1)"
if [[ "$?" -eq 0 ]]; then	#	Microsoft Word exists in /Applications.
	wordVersion="$(defaults read /Applications/Microsoft\ Word.app/Contents/Info CFBundleShortVersionString)"
	justDots="${wordVersion//[^.]}"
	[[ "${#justDots}" -ne "1" ]] && wordVersionMiddle=true || wordVersionMiddle=false
	wordVersionMajor="${wordVersion%%.*}"
	wordVersionMinor="${wordVersion##*.}"
	if $wordVersionMiddle || [[ "$wordVersionMajor" -lt "16" || "$wordVersionMinor" -lt "28" ]]; then
		printf "\nUnsupported Word version: $wordVersion. Only Office 2019/365 version 16.28 or newer is supported.\n\n"; exit
	fi
else
	printf "\nMicrosoft Word was not found in /Applications.\n\nOffice 2019/365 must be installed and should be updated before this script is run.\n\n"; exit
fi
#
#	Change the variables into the values used by defaults write.
#	Keep a count of what's changed in both plists.
numOfficeOptions=0
numAutoUpdateOptions=0
[[ "$diagnosticData" != "" ]] && { diagnosticData="$(echo ${diagnosticData:1:1} | tr "[:lower:]" "[:upper:]")${diagnosticData:2:${#diagnosticData}}DiagnosticData"; ((numOfficeOptions++)); }
[[ "$conOfficeExpers" != "" ]] && { conOfficeExpers="${conOfficeExpers#-con}"; ((numOfficeOptions++)); }
[[ "$analyzingContent" != "" ]] && { analyzingContent="${analyzingContent#-acp}"; ((numOfficeOptions++)); }
[[ "$downloadingContent" != "" ]] && { downloadingContent="${downloadingContent#-dcp}"; ((numOfficeOptions++)); }
[[ "$optConExpers" != "" ]] && { optConExpers="${optConExpers#-opt}"; ((numOfficeOptions++)); }
[[ "$MAU_HowToCheck" == "-manual" ]] && { MAU_HowToCheck="Manual"; ((numAutoUpdateOptions++)); }
[[ "$MAU_HowToCheck" == "-check" ]] && { MAU_HowToCheck="AutomaticCheck"; ((numAutoUpdateOptions++)); }
[[ "$MAU_HowToCheck" == "-install" ]] && { MAU_HowToCheck="AutomaticDownload"; ((numAutoUpdateOptions++)); }
[[ "$MAU_ReqDataNotice" == "-reqonly" ]] && { MAU_ReqDataNotice="RequiredDataOnly"; ((numAutoUpdateOptions++)); }
[[ "$MAU_ReqDataNotice" == "-reqopt" ]] && { MAU_ReqDataNotice="RequiredAndOptionalData"; ((numAutoUpdateOptions++)); }
#
#	Whew, finally good to go.
#
[[ $(($numOfficeOptions + $numAutoUpdateOptions)) -eq 2 ]] && plistPlural="s" || plistPlural=""			#	Whether one or both plists are set.
[[ numOfficeOptions -gt 1 || numAutoUpdateOptions -gt 1 ]] && keyValuePlural="s" || keyValuePlural=""	#	Whether one or more keys & values are set.
printf "\nThe following plist$plistPlural, key$keyValuePlural, and value$keyValuePlural will be set for "
if [[ "$setWho" == "me" ]] ; then
	printf "$myusername ($(dscl . -read /Users/$myusername RealName | sed -n 's/^ //g;2p'))"
elif [[ "$setWho" == "other" ]]; then
	printf "all users except Guest and $myusername ($(dscl . -read /Users/$myusername RealName | sed -n 's/^ //g;2p'))"
else
	printf "all users except Guest"
fi
printf ":\n"
[[ "$diagnosticData" != "" ]] && printf "\tcom.microsoft.office: DiagnosticDataTypePreference=\"$diagnosticData\"\n"
[[ "$conOfficeExpers" != "" ]] && printf "\tcom.microsoft.office: ConnectedOfficeExperiencesPreference=\"$conOfficeExpers\"\n"
[[ "$analyzingContent" != "" ]] && printf "\tcom.microsoft.office: OfficeExperiencesAnalyzingContentPreference=\"$analyzingContent\"\n"
[[ "$downloadingContent" != "" ]] && printf "\tcom.microsoft.office: OfficeExperiencesDownloadingContentPreference=\"$downloadingContent\"\n"
[[ "$optConExpers" != "" ]] && printf "\tcom.microsoft.office: OptionalConnectedExperiencesPreference=\"$optConExpers\"\n"
[[ "$MAU_HowToCheck" != "" ]] && printf "\tcom.microsoft.autoupdate2: HowToCheck=\"$MAU_HowToCheck\"\n"
[[ "$MAU_HowToCheck" == "Manual" && "$setWho" == "all" ]] && otherUsersBlurb=" (other users only)" || otherUsersBlurb=""
[[ "$MAU_HowToCheck" == "Manual" && "$setWho" != "me" ]] && printf \
		"\tcom.microsoft.autoupdate2: EnableCheckForUpdatesButton=\"false\"$otherUsersBlurb\n\tcom.microsoft.autoupdate2: StartDaemonOnAppLaunch=\"false\"$otherUsersBlurb\n"
[[ "$MAU_ReqDataNotice" != "" ]] && printf "\tcom.microsoft.autoupdate2: AcknowledgedDataCollectionPolicy=\"$MAU_ReqDataNotice\"\n"
printf "\n"
#
#	Loop through all users - even when setting only the current user.
dscl . -list /Users NFSHomeDirectory | awk '/[[:space:]]\/Users\//{print $1}' | \
while read username ; do
	if [[ "$username" != "Guest" ]] && \
			{ \
				[[ "$setWho" == "all" ]] || \
				[[ "$setWho" == "me"  && "$username" == "$myusername" ]] || \
				[[ "$setWho" == "other" && "$username" != "$myusername" ]]; \
			}; then
		[[ "$diagnosticData" != "" ]] && sudo -u "$username" defaults write com.microsoft.office DiagnosticDataTypePreference -string "$diagnosticData"
		[[ "$conOfficeExpers" != "" ]] && sudo -u "$username" defaults write com.microsoft.office ConnectedOfficeExperiencesPreference -string "$conOfficeExpers"
		[[ "$analyzingContent" != "" ]] && sudo -u "$username" defaults write com.microsoft.office OfficeExperiencesAnalyzingContentPreference -boolean "$analyzingContent"
		[[ "$downloadingContent" != "" ]] && sudo -u "$username" defaults write com.microsoft.office OfficeExperiencesDownloadingContentPreference -boolean "$downloadingContent"
		[[ "$optConExpers" != "" ]] && sudo -u "$username" defaults write com.microsoft.office OptionalConnectedExperiencesPreference -boolean "$optConExpers"
		if [[ "$MAU_HowToCheck" != "" ]]; then
			sudo -u "$username" defaults write com.microsoft.autoupdate2 HowToCheck -string "$MAU_HowToCheck"
			if [[ "$MAU_HowToCheck" == "Manual" && "$username" != "$myusername" ]]; then
				sudo -u "$username" defaults write com.microsoft.autoupdate2 EnableCheckForUpdatesButton -bool "false"
				sudo -u "$username" defaults write com.microsoft.autoupdate2 StartDaemonOnAppLaunch -bool "false"
			fi
		fi
		[[ "$MAU_ReqDataNotice" != "" ]] && sudo -u "$username" defaults write com.microsoft.autoupdate2 AcknowledgedDataCollectionPolicy -string "$MAU_ReqDataNotice"
		printf "%s\n" "Preferences set for $username ($(dscl . -read /Users/$username RealName | sed -n 's/^ //g;2p'))."
	fi
done
#
printf "\n"
#
######################################################################
#
#	The End
#
######################################################################
